# coding: UTF-8

require 'm2w_request'

#=コマンドの標準実装
#
# 最初の著者:: トゥイー
# リポジトリ情報:: $Id: standard_command.rb 620 2012-03-17 02:35:00Z toy_dev $
# 著作権:: Copyright (C) Ownway.info, 2011. All rights reserved.
# ライセンス:: CPL(Common Public Licence)
class StandardCommand

	attr_writer :weblog
	attr_writer :mail_parser
	attr_writer :mail_sender
	attr_writer :plugin_manager
	attr_writer :mail
	attr_writer :param

	attr_reader :report

	# public
	def run()
		M2W_LOGGER.debug("Start  #{self.class.name}#run")
		@report = {}

		if @weblog != nil then
			setup_weblog
		end

		run_before_parsing()
		request = parse_mail_content()
		begin
			return run_after_parsing(request)
		ensure
			M2W_LOGGER.debug("Finish #{self.class.name}#run")
		end
	end

	# overridable
	def setup_weblog
		@weblog.mail_parser = @mail_parser if @weblog.respond_to?("mail_parser=")
		@weblog.mail_sender = @mail_sender if @weblog.respond_to?("mail_sender=")
		@weblog.plugin_manager = @plugin_manager if @weblog.respond_to?("plugin_manager=")
	end

	# overridable
	def header_alias()
		return {}
	end

	# overridable
	def parse_mail_content
		header = nil
		content = nil
		contents = nil
		html_header = nil
		html_content = nil
		html_contents = nil

		if @mail.has_content then
			(header, content, contents) = @mail_parser.parse_content(@mail.content, header_alias(), Mail2WeblogMailParserPlainTextType.new)
		end

		if @mail.has_html_content then
			(html_header, html_content, html_contents) = @mail_parser.parse_content(@mail.html_content, header_alias(), Mail2WeblogMailParserHtmlTextType.new)
		end

		return Mail2WeblogRequest.new(
			@mail,
			header != nil ? header : html_header,
			content,
			contents,
			@mail.attachments,
			html_content,
			html_contents,
			@mail.related_attachments)
	end

	# overridable
	def run_before_parsing
	end

	# overridable
	def run_after_parsing(request)
		return true
	end

	def make_exception_contents(errors, warnings)
		result = ""

		if errors.length > 0 then
			result << "\n= エラー =\n\n"
			errors.each do |error|
				result << "*#{error.message}\n"
			end
		end

		if warnings.length > 0 then
			result << "\n= 警告 =\n\n"
			warnings.each do |warning|
				result << "*#{warning.message}\n"
			end
		end

		return result
	end

end
