# coding: UTF-8

require 'plugin/weblog/lib/attachment_media_poster'
require 'plugin/weblog/lib/related_media_poster'
require 'plugin/weblog/lib/standard_command'
require 'plugin/weblog/lib/weblog_plugin_error'
require 'plugin/weblog/lib/weblog_plugin_warning'

#=標準のブログ記事投稿用ウェブログプラグイン
#
# 最初の著者:: トゥイー
# リポジトリ情報:: $Id: standard_post.rb 621 2012-03-17 02:36:35Z toy_dev $
# 著作権:: Copyright (C) Ownway.info, 2011. All rights reserved.
# ライセンス:: CPL(Common Public Licence)
class StandardPost_WeblogPlugin < StandardCommand_WeblogPlugin

	def post(request, blogid)
		M2W_LOGGER.debug("Start  #{self.class.name}#post ... header = #{request.header.to_s}")

		client = @parent.client
		formatter = @parent.formatter
		postid = nil

		begin
			blogid = @parent.transform_blogid(blogid) if @parent.respond_to?('transform_blogid')

			before(client, request, blogid, @errors, @warnings)

			# ブログ記事を仮投稿する
			begin
				postid = post_temp_article(client, blogid)
			rescue => e
				M2W_LOGGER.error("ブログ記事の仮投稿に失敗しました。")
				M2W_LOGGER.error(e)
				@errors.push(WeblogPluginError.new(
					"ブログ記事仮投稿エラー(メッセージ = #{e.message})",
					"ブログ記事の仮投稿に失敗しました(メッセージ = #{e.message})。"))
				M2W_LOGGER.debug("Finish #{self.class.name}#post ... result = false")
				return false
			end

			vars = {
				:blogid => blogid,
				:encoding => @parent.encoding,
				M2W_FORMAT_PLUGIN_CONF_HEADER_NAME => request.header[M2W_FORMAT_PLUGIN_CONF_HEADER_NAME]
			}
			vars.merge!(request.header)

			# 書式プラグインの事前処理を行う
			formatter.preset(request, vars, @errors, @warnings)

			# 投稿日付を取得する
			dateCreated = @parent.parse_date(request.header['dateCreated'], Time.now, @errors, @warnings)
			if request.header['dateCreated'] != nil then
				request.header['dateCreated'] = @parent.get_date(dateCreated)
			end

			# ファイル転送の事前準備をする
			media_posters = []
			# 添付ファイルを投稿対象に追加する
			if request.attachments != nil then
				request.attachments.each_with_index do |attachment, i|
					attachment_index = i + 1
					media_name = @parent.get_attachment_filepath(postid, dateCreated, attachment_index, attachment.filename)
					thumbnail_name = @parent.get_thumbnail_filepath(postid, dateCreated, attachment_index, attachment.filename)
					media_posters.push(AttachmentMediaPoster.new(attachment, attachment_index, media_name, thumbnail_name, vars))
				end
			end
			# 関連ファイルを投稿対象に追加する
			if formatter.mode(request) == 'html' && request.related_attachments != nil then
				request.related_attachments.each_with_index do |related_attachment, i|
					related_attachment_index = i + 1
					media_name = "rel#{postid}_#{related_attachment_index}#{File.extname(related_attachment.filename)}"
					thumbnail_name = "relthumbs#{postid}_#{related_attachment_index}#{File.extname(related_attachment.filename)}"
					media_posters.push(RelatedMediaPoster.new(related_attachment, media_name, thumbnail_name, vars))
				end
			end

			# ファイルを転送する
			vars[:attachment_urls] = {}
			vars[:thumbnail_urls] = {}
			vars[:related_attachment_urls] = {}
			vars[:related_thumbnail_urls] = {}
			post_medias(client, @parent.username, @parent.password, blogid, media_posters, @errors, @warnings)

			# 書式変換を行う
			formatted_contents = nil
			begin
				formatted_contents = formatter.format(request, vars, @errors, @warnings)
				request.header.merge!(formatted_contents)
			rescue => e
				M2W_LOGGER.error("ブログ記事の書式変換に失敗しました。")
				M2W_LOGGER.error(e)
				@errors.push(WeblogPluginError.new(
					"書式変換エラー(メッセージ = #{e.message})",
					"ブログ記事の書式変換に失敗しました(メッセージ = #{e.message})。"))
				M2W_LOGGER.debug("Finish #{self.class.name}#post ... result = false")
				return false
			end

			# 書式変換した結果を再投稿する
			begin
				repost_formatted_article(client, blogid, postid, request.header, dateCreated)
			rescue => e
				M2W_LOGGER.error("ブログ記事の書式変換後の投稿に失敗しました。")
				M2W_LOGGER.error(e)
				@errors.push(WeblogPluginError.new(
					"書式変換後ブログ記事投稿エラー(メッセージ = #{e.message})",
					"ブログ記事の書式プラグイン適用に失敗しました。(メッセージ = #{e.message})。"))
				M2W_LOGGER.debug("Finish #{self.class.name}#post ... result = false")
				return false
			end

			@report[:original_content] = request.original_content
			@report[:original_html_content] = request.original_html_content
			@report[:postid] = postid
			@report[:attachment_urls] = vars[:attachment_urls]
			@report[:thumbnail_urls] = vars[:thumbnail_urls]
			@report[:title] = request.header['title']

			after(client, request, blogid, postid, @errors, @warnings)
		rescue => e
			M2W_LOGGER.error("ブログ記事の投稿に失敗しました。")
			M2W_LOGGER.error(e)
			@errors.push(WeblogPluginError.new(
				"ブログ記事投稿エラー(投稿ＩＤ = #{postid}, メッセージ = #{e.message})",
				"ブログ記事の投稿に失敗しました。仮投稿したブログ記事の削除を試みます(投稿ＩＤ = #{postid}, メッセージ = #{e.message})。"))
			begin
				if postid != nil then
					delete_temp_article(client, blogid, postid)
					@errors.push(WeblogPluginError.new(
						"仮投稿ブログ記事削除済み(投稿ＩＤ = #{postid})",
						"仮投稿ブログ記事の削除に成功しました。ブログ記事を再投稿するには投稿内容を再確認してください(投稿ＩＤ = #{postid})。"))
				end
			rescue => e
				M2W_LOGGER.error("ブログ記事の削除に失敗しました。")
				M2W_LOGGER.error(e)
				@errors.push(WeblogPluginError.new(
					"仮投稿ブログ記事削除エラー(投稿ＩＤ = #{postid}, メッセージ = #{e.message})",
					"仮投稿ブログ記事の削除に失敗しました。該当のブログ記事を手動で削除してください(投稿ＩＤ = #{postid}, メッセージ = #{e.message})。"))
			end

			M2W_LOGGER.debug("Finish #{self.class.name}#post ... result = false")
			return false
		end

		M2W_LOGGER.debug("Finish #{self.class.name}#post ... result = true")
		return true
	end

	def header_alias
		return M2W_WEBLOG_POST_HEADER_ALIAS
	end

	def before(client, request, blogid, errors, warnings)
	end

	def after(client, request, blogid, postid, errors, warnings)
	end

	def post_temp_article(client)
		raise WeblogPluginError.new(
			"未実装エラー",
			"実装されていません。")
	end

	def repost_formatted_article(client, blogid, postid, header, dateCreated)
		raise WeblogPluginError.new(
			"未実装エラー",
			"実装されていません。")
	end

	def delete_temp_article(client, blogid, postid)
		raise WeblogPluginError.new(
			"未実装エラー",
			"実装されていません。")
	end

	def parse_postid(source)
		raise WeblogPluginError.new(
			"未実装エラー",
			"実装されていません。")
	end

end
